#include <dpmi.h>
#include <stdio.h>
#include "video-graphics.h"
#include <sys/nearptr.h>
#include <sys/farptr.h>
#include <go32.h>

int HRES, VRES;


long getAdd(int mode)
{
	__dpmi_regs regs;
	unsigned long dosbuf, address=0;

	dosbuf = __tb & 0xFFFFF; /* dosbuf  a rea de transferncia */
	dosmemput(&address, 4, dosbuf+40); /*apaga bytes 40 a 43 dessa rea*/

	regs.x.ax = 0x4F01; /* VBE get mode info */
	regs.x.di = dosbuf & 0xF; /* LSB da rea de transferncia */
	regs.x.es = (dosbuf >> 4) & 0xFFFF; /*MSB da rea de transferncia*/
	regs.x.cx = mode; /*informao sobre o modo 105h*/
	__dpmi_int(0x10, &regs); /* DOS int 10h */
	if (regs.h.ah) {
	printf("Modo nao suportado"); } /* tratamento de erro, modo no suportado */

	dosmemget(dosbuf+40, 4, &address); /* l o endereo da memria vdeo */
	if (!address) {
	printf("Modo nao suportado"); } /* tratamento de erro, modo no suportado */
	return address;
}

char * enter_graphics(int mode, __dpmi_meminfo *map)
{	
	__dpmi_regs regs; /*__dpmi_regs  uma estrutura que representa os registos do CPU */

	regs.x.ax = 0x4F02; /* registo AX do CPU com valor 0x4f02 */
	regs.x.bx = mode; /* registo BX do CPU com valor 0x4105 */
	__dpmi_int(0x10, &regs); /* gera interrupo de software 0x10, activando o modo grfico */
	
	
	char *video_mem; /* apontador genrico para aceder  memria grfica */ 
	__djgpp_nearptr_enable(); /* permite acesso  memria fsica usando apontadores */

/* endereo fsico da memria vdeo, usar o valor correcto */
	//map->address = 0xDA000000;
	map->address = getAdd(mode);
/* tamanho do bloco de memria, usar apenas o necessrio */
	map->size = HRES*VRES;
	__dpmi_physical_address_mapping(map); /* video_mem contm o endereo genrico a usar */
	video_mem = (char *)(map->address + __djgpp_conventional_base);
	
	return video_mem;
}



void leave_graphics(__dpmi_meminfo map)
{
	__dpmi_regs regs; 
	regs.x.ax = 0x0003; /* registo AX do CPU com valor 0x03 */
	__dpmi_int(0x10, &regs); /* gera interrupo software 0x10, entrando no modo texto */

}

void set_pixel(int x, int y, int color, char *base)
{
	*(base+y*HRES+x) = color;
}

int get_pixel(int x, int y, char *base)
{
	return *(base+y*HRES+x) ;
}

void clear_screen(char color, char *base)
{
	int y,x;
	for( y = 0; y < VRES;++y)
	for( x = 0; x < HRES; ++x)
		set_pixel(x,y,color, base);

}

void draw_line(int xi, int yi, int xf, int yf, int color, char *base)
{
	int hor, ver;
	
	if(yi == yf)
		{for(hor=xi;hor<xf;++hor)
				set_pixel(hor, yi, color, base);}
	else if(xf==xi)
		{for(ver=yi;ver<yf;++ver)
			set_pixel(xi,ver,color,base);}
	else
	{
		int x2=xf, x1=xi, y2=yf, y1=yi;
		int slope;
	    int dx, dy, incE, incNE, d, x, y;
	       // Onde inverte a linha x1 > x2       
	    if (x1 > x2)
		{
	       draw_line(x2, y2, x1, y1,color,base);
	       return;
	    }        
	    dx = x2 - x1;
	    dy = y2 - y1;
	   
	    if (dy < 0)
		{            
		   slope = -1;
		   dy = -dy;
	    }
	   else
	    {            
		  slope = 1;
	    }
	  
	  // Constante de Bresenham
	   incE = 2 * dy;
	   incNE = 2 * dy - 2 * dx;
	   d = 2 * dy - dx;
	   y = y1;
	   
	   for (x = x1; x <= x2; x++)
	   {
		   set_pixel(x, y, color, base);
		   if (d <= 0)
		   {
			 d += incE;
		   }
		   else
		   {
			 d += incNE;
			 y += slope;
		   }
	   }
	
	}
}

void drawCharAt(char c, int x, int y, int fore_color,int back_color,
				int char_scale,char *video_base, char *table) 
{
	char *char_def = table + c*CHR_H; 
	
	int lin, col, sx, sy;
	int x_ori = x;
	
	
	for (lin = 0; lin < CHR_H; lin++) {
		for (col = 0; col < 8; col++) {
			if ((*(char_def+lin)) & (1 << (7-col)))
				for (sx = 0; sx<char_scale; sx++)
					for (sy = 0; sy<char_scale; sy++)
						set_pixel(x+sx, y+sy, fore_color, video_base);
			else
				for (sx = 0; sx<char_scale; sx++)
					for (sy = 0; sy<char_scale; sy++)
						set_pixel(x+sx, y+sy, back_color, video_base);
			x += char_scale;
		}
		y += char_scale;
		x = x_ori;
	}
}

void drawFrame(char* video_mem, char* local_table, int inix, int iniy, int limx, int limy)
{
	int x, y;
	for(x = inix; x < limx;x+=20)
	{
		drawCharAt('*',x,limy-25,LETTER_COLOR, COLOR, 2 ,video_mem, local_table);
		drawCharAt('*',x,0,LETTER_COLOR, COLOR, 2 ,video_mem, local_table);
	}
	
	for(y = iniy; y < limy; y+=20)
	{
		drawCharAt('*',0,y,LETTER_COLOR, COLOR, 2 ,video_mem, local_table);
		drawCharAt('*',limx-17,y,LETTER_COLOR, COLOR, 2 ,video_mem, local_table);
	}
}

void drawStringAt(char* s, int x_ori, int y_ori, int fore_color, int back_color, int char_scale, char* video_base, char *table) {
	int i=0;
	for(i=0; s[i]; i++) {
		drawCharAt(s[i], x_ori, y_ori, fore_color, back_color, char_scale, video_base, table);
		x_ori += 8*char_scale;
	}
}

void drawIntAt(int num, int x_ori, int y_ori, int fore_color, int back_color, int char_scale, char* video_base, char *table) {
	int cnt=0, k=num;
	
	while (k>0) {
		k/=10;
		cnt++;
	}
	
	x_ori=x_ori+8*char_scale*cnt;
	while (cnt>0) {
		cnt--;
		drawCharAt((char)(num%10+'0'), x_ori, y_ori, fore_color, back_color, char_scale, video_base, table);
		num/=10;
		x_ori-=8*char_scale;
	}
}

void drawPicture(int n, char* video_mem, char* local_table)		//funcao que imprime a forca, tendo em conta o numero de tentativas
{
	int x = 750, y=400,c, space, cont = 0;

	
	while(cont <= 10 - n)  //Enquanto cont for menor ou igual, a 10 menos o numero de tentativas
	{
		
		if(cont == 0) // 9 tentativas restantes
		{
			drawStringAt("#################", x, y,LETTER_COLOR, COLOR, 1 ,video_mem, local_table);
			space = 15;
		}
		
		if(cont == 1)	// 8 tentativas restantes
		{
			x = 814;
			for(c = 0; c < 10; c++)
			{
				y -= space;
				drawCharAt('#',x,y,LETTER_COLOR, COLOR, 1 ,video_mem, local_table);
			}
			space=10;
		}
		if(cont == 2) // 7 tentativas restantes
		{
			for(c=0; c < 5; c++)
			{
				x += space;
				drawCharAt('#',x,y,LETTER_COLOR, COLOR, 1 ,video_mem, local_table);
			}
			space = 12;
		}
		
		if(cont == 3)// 6 tentativas restantes
		{
			for(c = 0; c < 2; c++)
			{
				y += space;
				drawCharAt('#',x,y,LETTER_COLOR, COLOR, 1 ,video_mem, local_table);
			}
			space = 12;
		}
		
		if(cont == 4)// 5 tentativas restantes
		{
			x -= 3;
			y += space;
			drawCharAt('O',x,y,LETTER_COLOR, COLOR, 2 ,video_mem, local_table);
			x += 3;
			space = 25;
		}
		
		if(cont == 5)// 4 tentativas restantes
		{
			y += space;
			drawCharAt('|',x,y,LETTER_COLOR, COLOR, 1 ,video_mem, local_table);
			drawCharAt('|',x,y+12,LETTER_COLOR, COLOR, 1 ,video_mem, local_table);
		}
		
		if(cont == 6)// 3 tentativas restantes
		{
			drawCharAt('\\',x+10,y,LETTER_COLOR, COLOR, 1 ,video_mem, local_table);
		}
		
		if(cont == 7)// 2 tentativas restantes
		{
			drawCharAt('/',x-10,y,LETTER_COLOR, COLOR, 1 ,video_mem, local_table);
		}
		
		if(cont == 8)// 1 tentativas restantes
		{
			y+=space;
			drawCharAt('\\',x+10,y,LETTER_COLOR, COLOR, 1 ,video_mem, local_table);
		}
		if(cont == 9) //0 tentativas restantes, jogo perdido
		{
			drawCharAt('/',x-10,y,LETTER_COLOR, COLOR, 1 ,video_mem, local_table);
		}
		cont++;
	}
}

