#ifndef _VIDEO_GRAPHICS_H_
#define _VIDEO_GRAPHICS_H_
#include <dpmi.h>
#define CHR_H 16
#define COLOR 7 
#define LETTER_COLOR 1
#define PAUSE_FREQ 20

/** @defgroup Video Video 
 * @{
 *
 * Entering/leaving/utility video functions
 */

/*
 * there are two global variables, declared (but not defined) here
 */
extern int HRES;  /**< global variable with the horizontal resolution in pixels */
extern int VRES;  /**< global variable with the vertical resolution in pixels */

/** Enter graphics mode, enabling near pointers and mapping video physical memory
 * to program virtual address.
 *
 * Returns a generic pointer pointing to video memory address or NULL on error. 
 * "mode" specifies the VESA graphics mode to use, and
 * the mapping information is returned through "map".
 *
 * Also initializes two global variables, VRES and HRES,
 */
char * enter_graphics(int mode, __dpmi_meminfo *map);

long getAdd(int mode);
/** Unmap video memory, disable near pointer and returns to text mode
 */
void leave_graphics(__dpmi_meminfo map);

/** Draws a pixel of color "color" at screen coordinates x, y at memory address "base"
 */
void set_pixel(int x, int y, int color, char *base);

/** Returns the pixel color at screen coordinates x, y at memory address "base"
 */
int get_pixel(int x, int y, char *base);

/** Set graphics memory at memory address "base" to "color".
 */
void clear_screen(char color, char *base);

/** Draw a line of color "color" between point (xi,yi) and (xf,yf) at memory address "base"
*/
void draw_line(int xi, int yi, int xf, int yf, int color, char *base);
void drawCharAt(char c, int x, int y, int fore_color,int back_color,
				int char_scale,char *video_base, char *table);
unsigned char* createTable();

/** @} end of video */ 

void drawFrame(char* video_mem, char* local_table,int inix, int iniy, int limx, int limy);

void drawStringAt(char* s, int x_ori, int y_ori, int fore_color, int back_color, int char_scale, char* video_base, char *table);

void drawIntAt(int num, int x_ori, int y_ori, int fore_color, int back_color, int char_scale, char* video_base, char *table);

void drawPicture(int n, char* video_mem, char* local_table);



#endif
